import os, sys
from SCRIBES.info import core_plugin_folder
from imp import load_source
from LanguageMode import LanguageModeBase, indent_line, unindent_line

class PythonMode(LanguageModeBase):
  
  def __init__(self, editor):
    LanguageModeBase.__init__(self, editor)
    self.__signal_id_1 = editor.textview.connect("key-press-event", self.__key_press_event_cb)
  
  def __key_press_event_cb(self, textview, event):
    from gtk.keysyms import bracketleft, bracketright, Return, colon
    from gtk.gdk import CONTROL_MASK
    if self.word_completion_mode():
      return False
    if event.keyval == Return:
      return self.__return_cb()
    elif event.keyval == colon:
      self.__colon_cb()
    elif event.state & CONTROL_MASK and event.keyval == bracketleft:
      self.__prev_define()
      return True
    elif event.state & CONTROL_MASK and event.keyval == bracketright:
      self.__next_define()
      return True
    return False
  
  def __prev_define(self):
    textbuffer = self.editor.textbuffer
    from SCRIBES.cursor import get_cursor_iterator, move_view_to_cursor
    cursor_position = get_cursor_iterator(textbuffer)
    for i in reversed(range(0,  cursor_position.get_line())):
      start = textbuffer.get_iter_at_line(i)
      end = textbuffer.get_iter_at_line(i + 1)
      text = textbuffer.get_text(start, end).strip(' ').strip('\t')
      if text.startswith('def ') or text.startswith('class '):
        textbuffer.place_cursor(start)
        move_view_to_cursor(self.editor.textview)
        return
    return
  
  def __next_define(self):
    textbuffer = self.editor.textbuffer
    from SCRIBES.cursor import get_cursor_iterator, move_view_to_cursor
    cursor_position = get_cursor_iterator(textbuffer)
    for i in range(cursor_position.get_line() + 1, textbuffer.get_line_count() - 1):
      start = textbuffer.get_iter_at_line(i)
      end = textbuffer.get_iter_at_line(i + 1)
      text = textbuffer.get_text(start, end).strip(' ').strip('\t')
      if text.startswith('def ') or text.startswith('class '):
        textbuffer.place_cursor(start)
        move_view_to_cursor(self.editor.textview)
        return
    return
  
  def __return_cb(self):
    textbuffer = self.editor.textbuffer
    textview = self.editor.textview
    from SCRIBES.cursor import  get_cursor_line, get_cursor_iterator, get_cursor_index, move_view_to_cursor
    from SCRIBES.lines import get_text_on_line, free_line_below
    cur_line_num = get_cursor_line(textbuffer)
    text = get_text_on_line(textbuffer, cur_line_num)[:get_cursor_index(textbuffer)]
    tmp = text.strip(' ').strip('\t')
    _ = lambda x: tmp.startswith(x) and tmp[-1] == ':'
    __ = lambda x: tmp.startswith(x) or tmp == x.rstrip(' ')
    if _('class ') or _('def ') or _('if ') or _('for ') or _('while ') or _('elif ') or _('else') or _('with ') or _('try ') or _('except ') or _('finally '):
      free_line_below(textbuffer)
      next = textbuffer.get_iter_at_line(cur_line_num + 1)
      textbuffer.place_cursor(next)
      move_view_to_cursor(self.editor.textview)
      indent_line(textview, get_cursor_line(textbuffer))
      return True
    
    if __('return ') or __('yield ') or __('raise ') or __('break ') or __('continue ') or __('pass '):
      free_line_below(textbuffer)
      next = textbuffer.get_iter_at_line(cur_line_num + 1)
      textbuffer.place_cursor(next)
      unindent_line(textview, get_cursor_line(textbuffer))
      move_view_to_cursor(self.editor.textview)
      return True
    return False
  
  def __colon_cb(self):
    textbuffer = self.editor.textbuffer
    textview = self.editor.textview
    from SCRIBES.cursor import  get_cursor_line, get_cursor_iterator, get_cursor_index, move_view_to_cursor
    from SCRIBES.lines import get_text_on_line, free_line_below
    cur_line_num = get_cursor_line(textbuffer)
    text = get_text_on_line(textbuffer, cur_line_num)[:get_cursor_index(textbuffer) + 1]
    tmp = text.strip(' ').strip('\t')
    _ = lambda x: tmp.startswith(x)
    if _('else') or _('elif') or _('except') or _('finally'):
      cur_level = self.__get_indent_level(text)
      prev_level = self.__get_prev_indent_level(textbuffer)
      if cur_level > prev_level:
        unindent_line(textview, get_cursor_line(textbuffer))
    return
  
  def __get_indent_level(self, text):
    indent_level = 0
    indent_chars = (' ', '\t')
    for i in text:
      if i in indent_chars:
        indent_level += 1
      else:
        break
    return indent_level
  
  def __get_prev_indent_level(self, textbuffer):
    from SCRIBES.cursor import get_cursor_iterator, move_view_to_cursor
    cursor_position = get_cursor_iterator(textbuffer)
    for i in reversed(range(0,  cursor_position.get_line())):
      start = textbuffer.get_iter_at_line(i)
      end = textbuffer.get_iter_at_line(i + 1)
      text = textbuffer.get_text(start, end)
      head = text.strip(' ').strip('\t')
      _ = lambda x: head.startswith(x) and head[-2] == ':'
      if _('if ') or _('elif ') or _('try ') or _('for ') or _('while ') or _('except'):
        return self.__get_indent_level(text)
      elif _('def') or _('class'):
        break
    return -1
  
#  def destroy(self):
#    from SCRIBES.utils import delete_attributes, disconnect_signal
#    disconnect_signal(self.__signal_id_1, self.editor.textview)
#    delete_attributes(self)
#    self = None
#    del self
#    return


