from SCRIBES.info import core_plugin_folder
from imp import load_source

indent_mod = load_source('indent', core_plugin_folder + '/Indent/indent.py')
unindent_mod = load_source('unindent', core_plugin_folder + '/Indent/unindent.py')

indent_line = getattr(indent_mod, 'indent_line')
unindent_line = getattr(unindent_mod, 'unindent_line')

__all__ = ['LanguageModeBase', 'indent_line', 'unindent_line']

class LanguageModeBase(object):
  def __init__(self, editor):
    self.editor = editor
    self.__wcmanager = self.editor.store.get_object('WordCompletionManager')
    self.__connected_to_wcmanager = False
    self.__is_completion_mode = False
    self.__signal_id_1 = editor.textview.connect("key-press-event", self.__key_press_event_cb)
  
  def word_completion_mode(self):
    if self.__wcmanager == None:
      self.__wcmanager = self.editor.store.get_object('WordCompletionManager')
    if not self.__connected_to_wcmanager:
      self.__wcmanager.connect('match-found', self.__completion_mode)
      self.__wcmanager.connect('no-match-found', self.__not_completion_mode)
      self.__connected_to_wcmanager = True
    return self.__is_completion_mode
  
  def __completion_mode(self, *args):
    self.__is_completion_mode = True
    return
  
  def __not_completion_mode(self, *args):
    self.__is_completion_mode = False
    return
  
  def __key_press_event_cb(self, textview, event):
    from gtk.keysyms import BackSpace
    if self.word_completion_mode():
      return False
    if event.keyval == BackSpace:
      return self.__backspace_cb()
    return False
  
  def __backspace_cb(self):
    textbuffer = self.editor.textbuffer
    textview = self.editor.textview
    from SCRIBES.cursor import  get_cursor_line, get_cursor_iterator, get_cursor_index, move_view_to_cursor
    from SCRIBES.lines import get_text_on_line, free_line_below
    cur_line_num = get_cursor_line(textbuffer)
    text = get_text_on_line(textbuffer, cur_line_num)[:get_cursor_index(textbuffer)]
    indent_chars = (' ', '\t')
    if text == '':
      return False
    for c in text:
      if not c in indent_chars:
        return False
    unindent_line(textview, cur_line_num)
    return True
  
  def destroy(self):
    from SCRIBES.utils import delete_attributes, disconnect_signal
    props = dir(lang_mode)
    for p in props:
      if '__signal_id_' in p:
        disconnect_signal(getattr(lang_mode, p), lang_mode.editor)
    delete_attributes(self)
    self = None
    del self
    return
